@@ -21,10 +21,10 @@
 #include "game.h"
 #include "score.h"
 
-#include <KDebug>
-#include <KLocale>
+#include <KLocalizedString>
 
 #include <QApplication>
+#include <QDebug>
 #include <QFile>
 
 namespace Kigo {
@@ -33,7 +33,7 @@ class UndoCommand : public QUndoCommand
 {
     public:
 
-    enum MoveType { Stone, Passed, Resigned };
+    enum class MoveType { Stone, Passed, Resigned };
 
     UndoCommand(Player *player, MoveType moveType, const QString &undoStr)
     : QUndoCommand(undoStr), m_player(player), m_moveType(moveType)
@@ -51,14 +51,14 @@ class UndoCommand : public QUndoCommand
 Game::Game(QObject *parent)
     : QObject(parent)
     , m_currentMove(0), m_lastUndoIndex(0), m_currentPlayer(&m_blackPlayer)
-    , m_blackPlayer(Player::Black), m_whitePlayer(Player::White)
+    , m_blackPlayer(Player::Color::Black), m_whitePlayer(Player::Color::White)
     , m_komi(4.5), m_boardSize(19), m_fixedHandicap(5), m_consecutivePassMoveNumber(0)
     , m_gameFinished(false)
 {
-    connect(&m_process, SIGNAL(readyRead()), this, SLOT(readyRead()));
-    connect(&m_undoStack, SIGNAL(canRedoChanged(bool)), this, SIGNAL(canRedoChanged(bool)));
-    connect(&m_undoStack, SIGNAL(canUndoChanged(bool)), this, SIGNAL(canUndoChanged(bool)));
-    connect(&m_undoStack, SIGNAL(indexChanged(int)), this, SLOT(undoIndexChanged(int)));
+    connect(&m_process, &QProcess::readyRead, this, &Game::readyRead);
+    connect(&m_undoStack, &QUndoStack::canRedoChanged, this, &Game::canRedoChanged);
+    connect(&m_undoStack, &QUndoStack::canUndoChanged, this, &Game::canUndoChanged);
+    connect(&m_undoStack, &QUndoStack::indexChanged, this, &Game::undoIndexChanged);
 }
 
 Game::~Game()
@@ -72,25 +72,25 @@ bool Game::start(const QString &command)
     m_process.start(command.toLatin1());      // Start new process with provided command
     if (!m_process.waitForStarted()) {        // Blocking wait for process start
         m_response = "Unable to execute command: " + command;
-        kDebug() << m_response;
+        //qDebug() << m_response;
         return false;
     }
     m_engineCommand = command;
-    //kDebug() << "Game" << command << "started...";
+    ////qDebug() << "Game" << command << "started...";
 
     // Test if we started a GTP-compatible Go game
     m_process.write("name\n");
     m_process.waitForReadyRead();
-    QString response = m_process.readAllStandardOutput();
+    const QString response = m_process.readAllStandardOutput();
     if (response.isEmpty() || !response.startsWith(QLatin1String("="))) {
-        m_response = "Game did not respond to GTP command \"name\"";
-        kDebug() << m_response;
+        m_response = QLatin1String("Game did not respond to GTP command \"name\"");
+        //qDebug() << m_response;
         stop();
         return false;
     } else {
         m_engineName = m_response;
     }
-    //kDebug() << "Game is a GTP-compatible Go game";
+    //qDebug() << "Game is a GTP-compatible Go game";
 
     m_process.write("version\n");
     if (waitResponse()) {
@@ -113,7 +113,7 @@ bool Game::init()
         return false;
     }
 
-    //kDebug() << "Init game!";
+    //qDebug() << "Init game!";
 
     m_process.write("clear_board\n");
     if (waitResponse()) {
@@ -160,7 +160,7 @@ bool Game::init(const QString &fileName, int moveNumber)
         if (waitResponse()) {
             m_fixedHandicap = m_response.toInt();
         }
-        //kDebug() << "Loaded komi is" << m_komi << "and handicap is" << m_fixedHandicap;
+        //qDebug() << "Loaded komi is" << m_komi << "and handicap is" << m_fixedHandicap;
 
         m_consecutivePassMoveNumber = 0;
         m_currentMove = moveNumber;
@@ -247,7 +247,7 @@ bool Game::setFixedHandicap(int handicap)
             return false;
         }
     } else {
-        //kWarning() << "Handicap" << handicap << " not set, it is too high!";
+        //qWarning() << "Handicap" << handicap << " not set, it is too high!";
         return false;
     }
 }
@@ -285,7 +285,7 @@ bool Game::playMove(const Player &player, const Stone &stone, bool undoable)
 
     const Player *tmp = &player;
     if (!tmp->isValid()) {
-        //kDebug() << "Invalid player argument, using current player!";
+        //qDebug() << "Invalid player argument, using current player!";
         tmp = m_currentPlayer;
     }
 
@@ -317,30 +317,30 @@ bool Game::playMove(const Player &player, const Stone &stone, bool undoable)
         m_currentMove++;
 
         if (undoable) {                         // Do undo stuff if desired
-            Player *player;
+            Player *playerTemp;
             UndoCommand::MoveType moveType;
             QString undoStr;
             if (tmp->isWhite()) {
-                player = &m_whitePlayer;
+                playerTemp = &m_whitePlayer;
                 if (stone.isValid()) {
-                    moveType = UndoCommand::Stone;
+                    moveType = UndoCommand::MoveType::Stone;
                     undoStr = i18nc("%1 stone coordinate", "White %1", stone.toString());
                 } else {
-                    moveType = UndoCommand::Passed;
+                    moveType = UndoCommand::MoveType::Passed;
                     undoStr = i18n("White passed");
                 }
             } else {
-                player = &m_blackPlayer;
+                playerTemp = &m_blackPlayer;
                 if (stone.isValid()) {
-                    moveType = UndoCommand::Stone;
+                    moveType = UndoCommand::MoveType::Stone;
                     undoStr = i18nc("%1 stone coordinate", "Black %1", stone.toString());
                 } else {
-                    moveType = UndoCommand::Passed;
+                    moveType = UndoCommand::MoveType::Passed;
                     undoStr = i18n("Black passed");
                 }
             }
-            //kDebug() << "Push new undo command" << undoStr;
-            m_undoStack.push(new UndoCommand(player, moveType, undoStr));
+            //qDebug() << "Push new undo command" << undoStr;
+            m_undoStack.push(new UndoCommand(playerTemp, moveType, undoStr));
         }
 
         if (tmp->isWhite()) {                   // Determine the next current player
@@ -361,10 +361,9 @@ bool Game::generateMove(const Player &player, bool undoable)
     if (!isRunning()) {
         return false;
     }
-    bool boardChange = false;
     const Player *tmp = &player;
     if (!tmp->isValid()) {
-        //kDebug() << "Invalid player argument, using current player!";
+        //qDebug() << "Invalid player argument, using current player!";
         tmp = m_currentPlayer;
     }
 
@@ -378,16 +377,17 @@ bool Game::generateMove(const Player &player, bool undoable)
         m_process.write("genmove black\n");
     }
     if (waitResponse(true)) {
-        Player *player;
+        bool boardChange = false;
+        Player *playerTemp;
         UndoCommand::MoveType moveType;
         QString undoStr;
 
         if (tmp->isWhite()) {
-            player = &m_whitePlayer;
+            playerTemp = &m_whitePlayer;
         } else {
-            player = &m_blackPlayer;
+            playerTemp = &m_blackPlayer;
         }
-        if (m_response == "PASS") {
+        if (m_response == QLatin1String("PASS")) {
             m_currentMove++;
             emit passMovePlayed(*m_currentPlayer);
             if (m_consecutivePassMoveNumber > 0) {
@@ -395,16 +395,16 @@ bool Game::generateMove(const Player &player, bool undoable)
                 m_gameFinished = true;
             }
             m_consecutivePassMoveNumber++;
-            moveType = UndoCommand::Passed;
+            moveType = UndoCommand::MoveType::Passed;
             if (tmp->isWhite()) {
                 undoStr = i18n("White passed");
             } else {
                 undoStr = i18n("Black passed");
             }
-        } else if (m_response == "resign") {
+        } else if (m_response == QLatin1String("resign")) {
             emit resigned(*m_currentPlayer);
             m_gameFinished = true;
-            moveType = UndoCommand::Resigned;
+            moveType = UndoCommand::MoveType::Resigned;
             if (tmp->isWhite()) {
                 undoStr = i18n("White resigned");
             } else {
@@ -414,7 +414,7 @@ bool Game::generateMove(const Player &player, bool undoable)
             m_currentMove++;
             m_movesList.append(Move(tmp, Stone(m_response)));
             m_consecutivePassMoveNumber = 0;
-            moveType = UndoCommand::Stone;
+            moveType = UndoCommand::MoveType::Stone;
             if (tmp->isWhite()) {
                 undoStr = i18nc("%1 response from Go engine", "White %1", m_response);
             } else {
@@ -424,8 +424,8 @@ bool Game::generateMove(const Player &player, bool undoable)
         }
 
         if (undoable) {
-            //kDebug() << "Push new undo command" << undoStr;
-            m_undoStack.push(new UndoCommand(player, moveType, undoStr));
+            //qDebug() << "Push new undo command" << undoStr;
+            m_undoStack.push(new UndoCommand(playerTemp, moveType, undoStr));
         }
         if (tmp->isWhite()) {
             setCurrentPlayer(m_blackPlayer);
@@ -486,20 +486,20 @@ bool Game::redoMove()
 
     const UndoCommand *undoCmd = static_cast<const UndoCommand*>(m_undoStack.command(m_undoStack.index()));
 
-    Player *player = undoCmd->player();
+    const Player *player = undoCmd->player();
 
-    if (undoCmd->moveType() == UndoCommand::Passed) {
-        kDebug() << "Redo a pass move for" << player << undoCmd->text();
+    if (undoCmd->moveType() == UndoCommand::MoveType::Passed) {
+        //qDebug() << "Redo a pass move for" << player << undoCmd->text();
         playMove(*player, Stone(), false);         // E.g. pass move
-    } else if (undoCmd->moveType() == UndoCommand::Resigned) {
-        // Note: Altough it is possible to undo after a resign and redo it,
+    } else if (undoCmd->moveType() == UndoCommand::MoveType::Resigned) {
+        // Note: Although it is possible to undo after a resign and redo it,
         //       it is a bit questionable whether this makes sense logically.
-        kDebug() << "Redo a resign for" << player << undoCmd->text();
+        //qDebug() << "Redo a resign for" << player << undoCmd->text();
         emit resigned(*player);
         m_gameFinished = true;
         //emit resigned(*m_currentPlayer);
     } else {
-        kDebug() << "Redo a normal move for" << player << undoCmd->text();
+        //qDebug() << "Redo a normal move for" << player << undoCmd->text();
         playMove(*player, Stone(undoCmd->text()), false);
     }
     m_undoStack.redo();
@@ -509,9 +509,6 @@ bool Game::redoMove()
 Move Game::lastMove() const
 {
     Q_ASSERT(!m_movesList.isEmpty());
-    /*if (m_movesList.isEmpty())
-        return Move(m_currentPlayer, Stone());
-    else*/
     return m_movesList.last();
 }
 
@@ -603,7 +600,7 @@ QList<Stone> Game::bestMoves(const Player &player)
         m_process.write("top_moves_black\n");
     }
     if (waitResponse(true) && !m_response.isEmpty()) {
-        QStringList parts = m_response.split(' ');
+        const QStringList parts = m_response.split(' ');
         if (parts.size() % 2 == 0) {
             for (int i = 0; i < parts.size(); i += 2) {
                 list.append(Stone(parts[i], QString(parts[i + 1]).toFloat()));
@@ -650,38 +647,39 @@ int Game::captures(const Player &player)
 Game::FinalState Game::finalState(const Stone &stone)
 {
     if (!isRunning() || !stone.isValid()) {
-        return FinalStateInvalid;
+        return FinalState::FinalStateInvalid;
     }
 
     m_process.write("final_status " + stone.toLatin1() + '\n');
     if (waitResponse()) {
-        if (m_response == "alive") return FinalAlive;
-        else if (m_response == "dead") return FinalDead;
-        else if (m_response == "seki") return FinalSeki;
-        else if (m_response == "white_territory") return FinalWhiteTerritory;
-        else if (m_response == "blacK_territory") return FinalBlackTerritory;
-        else if (m_response == "dame") return FinalDame;
-        else return FinalStateInvalid;
-    } else
-        return FinalStateInvalid;
+        if (m_response == QLatin1String("alive")) { return FinalState::FinalAlive; }
+        else if (m_response == QLatin1String("dead")) { return FinalState::FinalDead; }
+        else if (m_response == QLatin1String("seki")) { return FinalState::FinalSeki; }
+        else if (m_response == QLatin1String("white_territory")) { return FinalState::FinalWhiteTerritory; }
+        else if (m_response == QLatin1String("blacK_territory")) { return FinalState::FinalBlackTerritory; }
+        else if (m_response == QLatin1String("dame")) { return FinalState::FinalDame; }
+        else { return FinalState::FinalStateInvalid; }
+    } else {
+        return FinalState::FinalStateInvalid;
+    }
 }
 
 QList<Stone> Game::finalStates(FinalState state)
 {
     QList<Stone> list;
-    if (!isRunning() || state == FinalStateInvalid) {
+    if (!isRunning() || state == FinalState::FinalStateInvalid) {
         return list;
     }
 
     QByteArray msg("final_status_list ");
     switch (state) {
-        case FinalAlive: msg.append("alive"); break;
-        case FinalDead: msg.append("dead"); break;
-        case FinalSeki: msg.append("seki"); break;
-        case FinalWhiteTerritory: msg.append("white_territory"); break;
-        case FinalBlackTerritory: msg.append("black_territory"); break;
-        case FinalDame: msg.append("dame"); break;
-        case FinalStateInvalid: /* Will never happen */ break;
+        case FinalState::FinalAlive: msg.append("alive"); break;
+        case FinalState::FinalDead: msg.append("dead"); break;
+        case FinalState::FinalSeki: msg.append("seki"); break;
+        case FinalState::FinalWhiteTerritory: msg.append("white_territory"); break;
+        case FinalState::FinalBlackTerritory: msg.append("black_territory"); break;
+        case FinalState::FinalDame: msg.append("dame"); break;
+        case FinalState::FinalStateInvalid: /* Will never happen */ break;
     }
     msg.append('\n');
     m_process.write(msg);
@@ -717,14 +715,14 @@ bool Game::waitResponse(bool nonBlocking)
 {
     if (m_process.state() != QProcess::Running) {   // No GTP connection means no computing fun!
         switch (m_process.error()) {
-            case QProcess::FailedToStart: m_response = "No Go game is running!"; break;
-            case QProcess::Crashed: m_response = "The Go game crashed!"; break;
-            case QProcess::Timedout: m_response = "The Go game timed out!"; break;
+            case QProcess::FailedToStart: m_response = QLatin1String("No Go game is running!"); break;
+            case QProcess::Crashed: m_response = QLatin1String("The Go game crashed!"); break;
+            case QProcess::Timedout: m_response = QLatin1String("The Go game timed out!"); break;
             case QProcess::WriteError: m_response = m_process.readAllStandardError(); break;
             case QProcess::ReadError: m_response = m_process.readAllStandardError(); break;
-            case QProcess::UnknownError: m_response = "Unknown error!"; break;
+            case QProcess::UnknownError: m_response = QLatin1String("Unknown error!"); break;
         }
-        kWarning() << "Command failed:" << m_response;
+        qWarning() << "Command failed:" << m_response;
         return false;
     }
 
@@ -770,7 +768,7 @@ void Game::gameSetup()
 {
     emit boardInitialized();
 }
-    
+
 void Game::readyRead()
 {
     m_waitAndProcessEvents = false;
@@ -778,22 +776,6 @@ void Game::readyRead()
 
 void Game::undoIndexChanged(int index)
 {
-    /*int diff = index - m_lastUndoIndex;
-
-    //TODO: Fix this ugly mess!
-    kDebug() << "Undo stack index changed by " << diff << "...";
-    // Bigger index changed usually mean that some QUndoView got a MouseClickEvent.
-    // This means we have to replay more undo/redo steps than normally. If the
-    // difference is greater than 1 we have to replay diff-1 steps.
-    if (diff > 1) {
-        for (int i = 0; i < diff; i++) {
-            redoMove();
-        }
-    } else if (diff < -1) {
-        for (int i = 0; i > diff; i--) {
-            undoMove();
-        }
-    }*/
     m_lastUndoIndex = index;
 }
 
@@ -804,5 +786,3 @@ void Game::setCurrentPlayer(Player &player)
 }
 
 } // End of namespace Kigo
-
-#include "moc_game.cpp"
